/* /////////////////////////////////////////////////////////////////////////////
 * File:        mmap_ex.c
 *
 * Purpose:     Implementation of the mmap_file function; platform-independent.
 *
 * Created      18th December 2003
 * Updated:     21st March 2004
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/
 *                          http://www.synesis.com.au/software/
 *
 *              email:      software@synesis.com.au
 *                          software@synesis-group.com
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#define _SYNSOFT_VER_C_MMAP_EX_MAJOR    1
#define _SYNSOFT_VER_C_MMAP_EX_MINOR    1
#define _SYNSOFT_VER_C_MMAP_EX_REVISION	2
#define _SYNSOFT_VER_C_MMAP_EX_EDIT     5

#include <stddef.h>

#include <fcntl.h>
#include <io.h>
#if !defined(SEEK_END) || \
    !defined(SEEK_SET)
# include <stdio.h>
#endif /* !SEEK_END || !SEEK_SET */
#include <share.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/stat.h>

#if defined(__BORLANDC__) || \
    defined(__DMC__)
#elif defined(__INTEL_COMPILER) || \
	  defined(__MWERKS__) || \
      defined(_MSC_VER)
# define lseek	_lseek
# define open	_open
# define write	_write
#else
# error Compiler not discriminated
#endif /* compiler */


void *mmap_file(char const  *fileName   /* = NULL */
            ,   size_t      len
			,	size_t		offset
            ,   int         prot
            ,   int         flags
			,	size_t		*plen)
{
    void    *pvMap;
    int     fd;
    int     oflag;
    int     pmode;

    flags = 0; /* Ignored for the current implentation */

    if( NULL == fileName ||
        PROT_WRITE == (prot & PROT_WRITE))
    {
        prot    =   PROT_READ | PROT_WRITE;
        oflag   =   O_CREAT | O_BINARY | O_RDWR;
        pmode   =   SH_DENYRW | S_IREAD | S_IWRITE;
    }
    else
    {
        prot    =   PROT_READ;
        oflag   =   O_BINARY | O_RDONLY;
        pmode   =   SH_DENYWR | S_IREAD;
    }

    if(NULL == fileName)
    {
        flags |= MAP_ANONYMOUS;

        if(0 == len)
        {
            return MAP_FAILED;
        }
        else
        {
            size_t  pagesize    =   getpagesize();

            len = (len + pagesize - 1) & ~(pagesize - 1);
        }

        fd = -1;
    }
    else
    {
        size_t  fileLen;

#if defined(__BORLANDC__)
        fd = open(fileName, oflag);

        ((void)pmode);
#else
        fd = open(fileName, oflag, pmode);
#endif /* __SYNSOFT_DBS_DEBUG */

        if(fd < 0)
        {
            return MAP_FAILED;
        }

        fileLen = lseek(fd, 0, SEEK_END);
        lseek(fd, 0, SEEK_SET);

        if(0 == len)
        {
            len = fileLen;
        }
        else
        {
            if(prot & PROT_WRITE)
            {
                if(fileLen < len)
                {
                    lseek(fd, len - 1, SEEK_SET);
                    write(fd, "", 1);
                }
            }
        }
    }

    pvMap = mmap(NULL, len, prot, flags, fd, offset);

    if(-1 != fd)
    {
        _close(fd);
    }

	if(NULL != plen)
	{
		*plen = len;
	}

    return pvMap;
}

/* ////////////////////////////////////////////////////////////////////////// */
