/* /////////////////////////////////////////////////////////////////////////////
 * File:        atomic.h
 *
 * Purpose:     Declaration of Linux like atomic functions for the
 *              Win32 platform.
 *
 * Created      21st November 2003
 * Updated:     15th May 2004
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2004, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/
 *                          http://www.synesis.com.au/software/
 *
 *              email:      software@synesis.com.au
 *                          software@synesis-group.com
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _SYNSOFT_INCL_H_DIRENT
#define _SYNSOFT_INCL_H_ATOMIC

#ifndef _SYNSOFT_DOCUMENTATION_SKIP_SECTION
# define _SYNSOFT_VER_H_ATOMIC_MAJOR    1
# define _SYNSOFT_VER_H_ATOMIC_MINOR    1
# define _SYNSOFT_VER_H_ATOMIC_REVISION	2
# define _SYNSOFT_VER_H_ATOMIC_EDIT     5
#endif /* !_SYNSOFT_DOCUMENTATION_SKIP_SECTION */

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef WIN32
# error This file is only currently defined for Win32 compilation units
#endif /* WIN32 */

/* /////////////////////////////////////////////////////////////////////////////
 * Constants and definitions
 */


/* /////////////////////////////////////////////////////////////////////////////
 * Typedefs
 */

typedef signed long		atomic_t;

/* /////////////////////////////////////////////////////////////////////////////
 * API functions
 */

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define ATOMIC_INIT(i)		(i)

void    atomic_set(atomic_t volatile *v, int i);
int     atomic_write(atomic_t volatile *v, int i);
int     atomic_read(atomic_t volatile *v);

void    atomic_add(int i, atomic_t volatile *v);
void    atomic_sub(int i, atomic_t volatile *v);

void    atomic_inc(atomic_t volatile *v);
void    atomic_dec(atomic_t volatile *v);

// Returns the previous value
int     atomic_inc_and_test(atomic_t volatile *v);
// Returns the previous value
int     atomic_dec_and_test(atomic_t volatile *v);

// Returns the previous value
int     atomic_add_and_test(int i, atomic_t volatile *v);
// Returns the previous value
int     atomic_sub_and_test(int i, atomic_t volatile *v);


#ifdef __cplusplus
}
#endif /* __cplusplus */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _SYNSOFT_INCL_H_ATOMIC */

/* ////////////////////////////////////////////////////////////////////////// */
