/* /////////////////////////////////////////////////////////////////////////////
 * File:        time.h (sys/time.h)
 *
 * Purpose:     Declaration of the gettimeofday() API function and supporting
 *              types for the Win32 platform.
 *
 * Created      8th December 2002
 * Updated:     16th June 2004
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2004, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/
 *                          http://www.synesis.com.au/software/
 *
 *              email:      software@synesis.com.au
 *                          software@synesis-group.com
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/** \file sys/time.h
 *
 * Contains time functions and types.
 */

#ifndef _SYNSOFT_INCL_H_SYS_TIME
#define _SYNSOFT_INCL_H_SYS_TIME

#ifndef _SYNSOFT_DOCUMENTATION_SKIP_SECTION
# define _SYNSOFT_VER_H_SYS_TIME_MAJOR      1
# define _SYNSOFT_VER_H_SYS_TIME_MINOR      0
# define _SYNSOFT_VER_H_SYS_TIME_REVISION   4
# define _SYNSOFT_VER_H_SYS_TIME_EDIT       7
#endif /* !_SYNSOFT_DOCUMENTATION_SKIP_SECTION */

/* ////////////////////////////////////////////////////////////////////////// */

/** \weakgroup unixem Synesis Software UNIX Emulation for Win32
 * \brief The UNIX emulation library
 */

/** \weakgroup unixem_time time() API
 * \ingroup UNIXem unixem
 * \brief Time functions and types
 * @{
 */

/* ////////////////////////////////////////////////////////////////////////// */

#ifdef __cplusplus
extern "C"
{
#endif /* __cplusplus */

/** \brief A representation of current time, used by gettimeofday().
 *
 * This structure represents the time, in seconds and microseconds, since Jan.
 * 1, 1970, in UTC. Returned by the gettimeofday() function.
 */

#if !defined(_WINSOCKAPI_) && \
	!defined(_WINSOCK2API_) && \
	!defined(_WINSOCK2_H) && \
	!defined(_TIMEVAL_DEFINED)

/*
	(	!defined(__GNUC__) || \
		!defined(_TIMEVAL_DEFINED))
*/

struct timeval
{
    long tv_sec;    /*!< The number of seconds, since Jan. 1, 1970, in the time value. */
    long tv_usec;   /*!< The number of microseconds in the time value. */
};

# if !defined(_TIMEVAL_DEFINED)
#  define _TIMEVAL_DEFINED
# endif /* __GNUC__ */

#endif /* _WINSOCKAPI_ / _TIMEVAL_DEFINED */

/** \brief
 *
 * This structure is NOT used in any meaningful way by the gettimeofday() function
 * implementation in this library.
 */
struct timezone
{
    int tz_minuteswest; /*!< minutes west of Greenwich. */
    int tz_dsttime;     /*!< type of dst corrections to apply. */
};

/** \brief Accesses the current UTC time
 *
 * Retrieves the current time and timezone information, relative to Greenwich Mean Time
 *
 * \param tv Pointer to a timeval structure to receive the time information
 * \param tz Pointer to a timezone structure to receive the timezone information
 */
extern void gettimeofday(struct timeval *tv, struct timezone *tz);

#ifdef __cplusplus
} // extern "C"
#endif /* __cplusplus */

/* ////////////////////////////////////////////////////////////////////////// */

/** @} // end of group unixem_time */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _SYNSOFT_INCL_H_SYS_TIME */

/* ////////////////////////////////////////////////////////////////////////// */
