/* /////////////////////////////////////////////////////////////////////////////
 * File:        time.c
 *
 * Purpose:     gettimeofday() for the Win32 platform.
 *
 * Created      1st November 2003
 * Updated:     23rd May 2004
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2004, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/
 *                          http://www.synesis.com.au/software/
 *
 *              email:      software@synesis.com.au
 *                          software@synesis-group.com
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#define _SYNSOFT_VER_C_TIME_MAJOR       1
#define _SYNSOFT_VER_C_TIME_MINOR       0
#define _SYNSOFT_VER_C_TIME_REVISION    7
#define _SYNSOFT_VER_C_TIME_EDIT        7

struct timezone;

#include <windows.h>

/* #include "sys/time.h" */

/* /////////////////////////////////////////////////////////////////////////////
 * API functions
 */

/** This function is from the STLSoft libraries */
static long FILETIMEToUNIXTime(FILETIME const *ft, long *microseconds)
{
#if defined(__COMO__)
    long long   i;
#else /* ? compiler */
    LONGLONG    i;
#endif /* compiler */

    i = ft->dwHighDateTime;
    i <<= 32;
    i |= ft->dwLowDateTime;

#if (   defined(__DMC__) || \
        defined(__COMO__) || \
        defined(__GNUC__) || \
        defined(__MWERKS__))
    i -= 116444736000000000LL;
#else
    i -= 116444736000000000L;
#endif /* compiler */
    if(NULL != microseconds)
    {
        *microseconds = (long)((i % 10000000) / 10);
    }
    i /= 10000000;

    return (long)i;
}

void gettimeofday(struct timeval *tv, struct timezone *tz)
{
    SYSTEMTIME  st;
    FILETIME    ft;

    ((void)tz);

    GetLocalTime(&st);
    SystemTimeToFileTime(&st, &ft);

    tv->tv_sec = FILETIMEToUNIXTime(&ft, &tv->tv_usec);
}

/* ////////////////////////////////////////////////////////////////////////// */
